//===================================================================
//                                                                  =
// Author:    Martin Berriman                                       =
// Copyright: Innovatronics Ltd 2006                                =
//                                                                  =
//===================================================================

#ifndef _COMMSC
#define _COMMSC

// HEADERS ==========================================================
#include "comms.h"
// END HEADERS ======================================================

// C FILES ==========================================================
// END C FILES ======================================================

// MACROS =========================================================
// END MACROS =====================================================

// GLOBALS ==========================================================
// END GLOBALS ======================================================

//----------------------------------------------------------------------------------
void interpret_command(unsigned int8 command)
//----------------------------------------------------------------------------------
{
	switch(command)
	{
		case CMD_HW_VER:			get_hw_ver(); break;
		case CMD_SW_VER:			get_sw_ver(); break;
		case CMD_NUM_CHANNELS:		get_num_channels(); break;
		case CMD_GET_MODE:			get_mode(rx_str[1]); break;
		case CMD_GET_PRESSURE:		get_pressure(rx_str[1]); break;
		case CMD_GET_RESULT:		get_result(rx_str[1]); break;
		case CMD_GET_CAL_DATE:		get_cal_date(); break;
		case CMD_GET_SERIAL_NUM:	get_serial_num(); break;
		case CMD_MAX_PRESSURE:		get_max_pressure(); break;
		case CMD_MIN_PRESSURE:		get_min_pressure(); break;
		case CMD_LIM_PRESSURE:		get_lim_pressure(); break;	
		case CMD_STD_STRING:		get_std_string(); break;
		case CMD_UNITS_STRING:		get_units_string(); break;
		case CMD_CALIBRATE:			do_calibrate(); break;
		case CMD_SET_CAL_DATE:		set_cal_date(); break;
		case CMD_SET_SERIAL_NUM:	set_serial_num(); break;
		case CMD_ABORT:				do_abort(rx_str[1]); break;
		case CMD_SET_STANDARD:		set_standard(rx_str[1]); break;
		case CMD_SET_UNITS:			set_units(rx_str[1]); break;
		case CMD_DECIMAL_PLACES:	get_decimal_places(); break;
		case CMD_START_TEST:		do_start_test(rx_str[1]); break;
		case CMD_MAX_SAMPLES:		get_max_samples(); break;
		case CMD_GET_MAX_STAB_SECS:	get_max_stabilisation_seconds(); break;
		case CMD_GET_STAB_SECS:		get_stabilisation_seconds(rx_str[1]); break;
		case CMD_SET_CUSTOM_PARAMS:	set_custom_parameters(); break;
		case CMD_GET_CUSTOM_PARAMS:	get_custom_parameters(); break;
	}
}

//----------------------------------------------------------------------------------
void get_hw_ver(void)
//----------------------------------------------------------------------------------
{
	unsigned int16 ver;

	ver = HWVer;
	tempStr[0] = CMD_HW_VER;
	tempStr[1] = (unsigned char) MAKE8(ver, 1);
	tempStr[2] = (unsigned char) MAKE8(ver, 0);
	rs232_write_bytes(&tempStr[0], 3);
}

//----------------------------------------------------------------------------------
void get_sw_ver(void)
//----------------------------------------------------------------------------------
{
	unsigned int16 ver;

	ver = SWVer;
	tempStr[0] = CMD_SW_VER;
	tempStr[1] = (unsigned char) MAKE8(ver, 1);
	tempStr[2] = (unsigned char) MAKE8(ver, 0);
	rs232_write_bytes(&tempStr[0], 3);
}

//----------------------------------------------------------------------------------
void get_num_channels(void)
//----------------------------------------------------------------------------------
{
	tempStr[0] = CMD_NUM_CHANNELS;
	if (!dual_channel)
		tempStr[1] = 1;
	else
		tempStr[1] = 2;
	rs232_write_bytes(&tempStr[0], 2);
}

//----------------------------------------------------------------------------------
void get_mode(int8 channel)
//----------------------------------------------------------------------------------
{
	tempStr[0] = CMD_GET_MODE;
	tempStr[1] = test_state[channel];
	rs232_write_bytes(&tempStr[0], 2);
}

//----------------------------------------------------------------------------------
void get_pressure(int8 channel)
//----------------------------------------------------------------------------------
{
	tempStr[0] = CMD_GET_PRESSURE;
	if (test_state[channel] == state_complete)
	{
		tempStr[1] = (unsigned char) MAKE8(final_pressure[channel], 1);
		tempStr[2] = (unsigned char) MAKE8(final_pressure[channel], 0);
	}
	else
	{
		tempStr[1] = (unsigned char) MAKE8(pressure[channel], 1);
		tempStr[2] = (unsigned char) MAKE8(pressure[channel], 0);
	}
	rs232_write_bytes(&tempStr[0], 3);
}

//----------------------------------------------------------------------------------
void get_result(int8 channel)
//----------------------------------------------------------------------------------
{
	bool result;

	tempStr[0] = CMD_GET_RESULT;
	switch (channel)
	{
		case 0: result = test_result1; break;
		case 1: result = test_result2; break;
	}

	if (result)
		tempStr[1] = 1;
	else
		tempStr[1] = 0;

	rs232_write_bytes(&tempStr[0], 2);
}

//----------------------------------------------------------------------------------
void get_cal_date(void)
//----------------------------------------------------------------------------------
{
	tempStr[0] = CMD_GET_CAL_DATE;
	strncpy(&tempStr[1], &cal_date[0], 7);
	rs232_write_bytes(&tempStr[0], 8);
}

//----------------------------------------------------------------------------------
void get_serial_num(void)
//----------------------------------------------------------------------------------
{
	tempStr[0] = CMD_GET_SERIAL_NUM;
	tempStr[1] = (unsigned char) MAKE8(serial_no, 1);
	tempStr[2] = (unsigned char) MAKE8(serial_no, 0);
	rs232_write_bytes(&tempStr[0], 3);
}

//----------------------------------------------------------------------------------
void get_max_pressure(void)
//----------------------------------------------------------------------------------
{
	unsigned int16 val;

	switch (units)
	{
		case st_units_pa:	 val = test_standard_info.max_graph_pa; break;
		case st_units_mbar:  val = test_standard_info.max_graph_mbar; break;
		case st_units_mmh2o: val = test_standard_info.max_graph_mmh2o; break;
		case st_units_inh2o: val = test_standard_info.max_graph_inh2o; break;
	}

	tempStr[0] = CMD_MAX_PRESSURE;
	tempStr[1] = (unsigned char) MAKE8(val, 1);
	tempStr[2] = (unsigned char) MAKE8(val, 0);
	rs232_write_bytes(&tempStr[0], 3);
}

//----------------------------------------------------------------------------------
void get_min_pressure(void)
//----------------------------------------------------------------------------------
{
	unsigned int16 val;

	switch (units)
	{
		case st_units_pa:	 val = test_standard_info.min_graph_pa; break;
		case st_units_mbar:  val = test_standard_info.min_graph_mbar; break;
		case st_units_mmh2o: val = test_standard_info.min_graph_mmh2o; break;
		case st_units_inh2o: val = test_standard_info.min_graph_inh2o; break;
	}

	tempStr[0] = CMD_MIN_PRESSURE;
	tempStr[1] = (unsigned char) MAKE8(val, 1);
	tempStr[2] = (unsigned char) MAKE8(val, 0);
	rs232_write_bytes(&tempStr[0], 3);
}

//----------------------------------------------------------------------------------
void get_lim_pressure(void)
//----------------------------------------------------------------------------------
{
	unsigned int16 val;

	val = test_standard_info.limit;
	val = convert_units(val);

	tempStr[0] = CMD_LIM_PRESSURE;
	tempStr[1] = (unsigned char) MAKE8(val, 1);
	tempStr[2] = (unsigned char) MAKE8(val, 0);
	rs232_write_bytes(&tempStr[0], 3);
}

//----------------------------------------------------------------------------------
void get_std_string(void)
//----------------------------------------------------------------------------------
{
	char name[13];
	char * ptr1;
	char * ptr2;
	int8 len;

	strcpy(name, test_standard_info.name);
	ptr1 = &name[0];

	while (*ptr1 != '#')
	{
		ptr1++;
	}
	*ptr1 = '\0';

	len = strlen(name);

	ptr1 = &name[0];
	ptr2 = &tempStr[2];

	while (*ptr1 != '\0')
	{
		*ptr2++ = *ptr1++;
	}

	tempStr[0] = CMD_STD_STRING;
	tempStr[1] = len;

	rs232_write_bytes(&tempStr[0], len + 2);
}

//----------------------------------------------------------------------------------
void get_units_string(void)
//----------------------------------------------------------------------------------
{
	int8   offset;
	char * ptr;
	char   data;
	int8   num_bytes;

	tempStr[0] = CMD_UNITS_STRING;
	num_bytes = 1;

	ptr = &tempStr[2];
	offset = gUnitsOffsets[units];
	
    #ignore_warnings 201  //assignment within relational test 
	while (data = gUnitsStrings[offset++])
    {
      	*ptr++ = data;
		num_bytes++;
    }
	#ignore_warnings NONE

	tempStr[1] = num_bytes;
	num_bytes++;

	rs232_write_bytes(&tempStr[0], num_bytes);
}

//----------------------------------------------------------------------------------
void do_calibrate(void)
//----------------------------------------------------------------------------------
{
	tempStr[0] = CMD_CALIBRATE;
	tempStr[1] = 1;
	rs232_write_bytes(&tempStr[0], 2);

	testing = false;
	calibrating = true;
	calibrate();
}

//----------------------------------------------------------------------------------
void set_cal_date(void)
//----------------------------------------------------------------------------------
{
	tempStr[0] = CMD_SET_CAL_DATE;
	tempStr[1] = 1;
	rs232_write_bytes(&tempStr[0], 2);

	strncpy(&cal_date[0], &rx_str[1], 7);
	save_calibration_info();
}

//----------------------------------------------------------------------------------
void set_serial_num(void)
//----------------------------------------------------------------------------------
{
	tempStr[0] = CMD_SET_SERIAL_NUM;
	tempStr[1] = 1;
	rs232_write_bytes(&tempStr[0], 2);

	serial_no = MAKE16(rx_str[1], rx_str[2]);
	save_calibration_info();
}

//----------------------------------------------------------------------------------
void do_abort(int8 channel)
//----------------------------------------------------------------------------------
{
	tempStr[0] = CMD_ABORT;
	tempStr[1] = 1;
	rs232_write_bytes(&tempStr[0], 2);

	test_state[channel] = state_idle;
	reset_solenoids(channel);

	if ((test_state[0] == state_idle) && (test_state[1] == state_idle))
	{
		testing = false;
	}
}

//----------------------------------------------------------------------------------
void set_custom_parameters(void)
//----------------------------------------------------------------------------------
{
	STANDARD_TYPE standard_info;

	standard_info.name[0] = rx_str[1];
	standard_info.name[1] = rx_str[2];
	standard_info.name[2] = rx_str[3];
	standard_info.name[3] = rx_str[4];
	standard_info.name[4] = rx_str[5];
	standard_info.name[5] = rx_str[6];
	standard_info.name[6] = rx_str[7];
	standard_info.name[7] = rx_str[8];
	standard_info.name[8] = rx_str[9];
	standard_info.name[9] = rx_str[10];
	standard_info.name[10] = rx_str[11];
	standard_info.name[11] = rx_str[12];
	standard_info.name[12] = rx_str[13];
	standard_info.max_graph_pa = make16(rx_str[14], rx_str[15]);
	standard_info.min_graph_pa = make16(rx_str[16], rx_str[17]);
	standard_info.max_graph_mbar = make16(rx_str[18], rx_str[19]);
	standard_info.min_graph_mbar = make16(rx_str[20], rx_str[21]);
	standard_info.max_graph_mmh2o = make16(rx_str[22], rx_str[23]);
	standard_info.min_graph_mmh2o = make16(rx_str[24], rx_str[25]);
	standard_info.max_graph_inh2o = make16(rx_str[26], rx_str[27]);
	standard_info.min_graph_inh2o = make16(rx_str[28], rx_str[29]);
	standard_info.limit = make16(rx_str[30], rx_str[31]);
	standard_info.total_samples = make16(rx_str[32], rx_str[33]);
	standard_info.fast_fill_pa = make16(rx_str[34], rx_str[35]);
	standard_info.slow_fill_pa = make16(rx_str[36], rx_str[37]);
	standard_info.stabilise_pa = make16(rx_str[38], rx_str[39]);
	standard_info.test_pressure_pa = make16(rx_str[40], rx_str[41]);
	standard_info.stabilise_time_seconds = make16(rx_str[42], rx_str[43]);
	standard_info.stabilise_hysteresis_pa = rx_str[44];
	standard_info.stabilise_dwell_seconds = rx_str[45];
	standard_info.test_deflate_pause_seconds = rx_str[46];

	write_eeprom_custom_params(&standard_info);

	tempStr[0] = CMD_SET_CUSTOM_PARAMS;
	tempStr[1] = 1;
	rs232_write_bytes(&tempStr[0], 2);
}

//----------------------------------------------------------------------------------
void get_custom_parameters(void)
//----------------------------------------------------------------------------------
{
	STANDARD_TYPE standard_info;

	read_eeprom_custom_params(&standard_info);

	tempStr[0] = CMD_GET_CUSTOM_PARAMS;
	tempStr[1] = standard_info.name[0];
	tempStr[2] = standard_info.name[1];
	tempStr[3] = standard_info.name[2];
	tempStr[4] = standard_info.name[3];
	tempStr[5] = standard_info.name[4];
	tempStr[6] = standard_info.name[5];
	tempStr[7] = standard_info.name[6];
	tempStr[8] = standard_info.name[7];
	tempStr[9] = standard_info.name[8];
	tempStr[10] = standard_info.name[9];
	tempStr[11] = standard_info.name[10];
	tempStr[12] = standard_info.name[11];
	tempStr[13] = standard_info.name[12];
	
	tempStr[14] = (unsigned char) MAKE8(standard_info.max_graph_pa, 1);
	tempStr[15] = (unsigned char) MAKE8(standard_info.max_graph_pa, 0);

	tempStr[16] = (unsigned char) MAKE8(standard_info.min_graph_pa, 1);
	tempStr[17] = (unsigned char) MAKE8(standard_info.min_graph_pa, 0);

	tempStr[18] = (unsigned char) MAKE8(standard_info.max_graph_mbar, 1);
	tempStr[19] = (unsigned char) MAKE8(standard_info.max_graph_mbar, 0);

	tempStr[20] = (unsigned char) MAKE8(standard_info.min_graph_mbar, 1);
	tempStr[21] = (unsigned char) MAKE8(standard_info.min_graph_mbar, 0);

	tempStr[22] = (unsigned char) MAKE8(standard_info.max_graph_mmh2o, 1);
	tempStr[23] = (unsigned char) MAKE8(standard_info.max_graph_mmh2o, 0);

	tempStr[24] = (unsigned char) MAKE8(standard_info.min_graph_mmh2o, 1);
	tempStr[25] = (unsigned char) MAKE8(standard_info.min_graph_mmh2o, 0);

	tempStr[26] = (unsigned char) MAKE8(standard_info.max_graph_inh2o, 1);
	tempStr[27] = (unsigned char) MAKE8(standard_info.max_graph_inh2o, 0);

	tempStr[28] = (unsigned char) MAKE8(standard_info.min_graph_inh2o, 1);
	tempStr[29] = (unsigned char) MAKE8(standard_info.min_graph_inh2o, 0);

	tempStr[30] = (unsigned char) MAKE8(standard_info.limit, 1);
	tempStr[31] = (unsigned char) MAKE8(standard_info.limit, 0);

	tempStr[32] = (unsigned char) MAKE8(standard_info.total_samples, 1);
	tempStr[33] = (unsigned char) MAKE8(standard_info.total_samples, 0);

	tempStr[34] = (unsigned char) MAKE8(standard_info.fast_fill_pa, 1);
	tempStr[35] = (unsigned char) MAKE8(standard_info.fast_fill_pa, 0);

	tempStr[36] = (unsigned char) MAKE8(standard_info.slow_fill_pa, 1);
	tempStr[37] = (unsigned char) MAKE8(standard_info.slow_fill_pa, 0);

	tempStr[38] = (unsigned char) MAKE8(standard_info.stabilise_pa, 1);
	tempStr[39] = (unsigned char) MAKE8(standard_info.stabilise_pa, 0);

	tempStr[40] = (unsigned char) MAKE8(standard_info.test_pressure_pa, 1);
	tempStr[41] = (unsigned char) MAKE8(standard_info.test_pressure_pa, 0);

	tempStr[42] = (unsigned char) MAKE8(standard_info.stabilise_time_seconds, 1);
	tempStr[43] = (unsigned char) MAKE8(standard_info.stabilise_time_seconds, 0);

	tempStr[44] = (unsigned char) standard_info.stabilise_hysteresis_pa;
	tempStr[45] = (unsigned char) standard_info.stabilise_dwell_seconds;
	tempStr[46] = (unsigned char) standard_info.test_deflate_pause_seconds;

	rs232_write_bytes(&tempStr[0], 47);
}

//----------------------------------------------------------------------------------
void set_standard(valid_standards standard)
//----------------------------------------------------------------------------------
{
	if (standard < std_max)
	{
		tempStr[0] = CMD_SET_STANDARD;
		tempStr[1] = 1;
		rs232_write_bytes(&tempStr[0], 2);

		test_standard_number = standard;
		save_calibration_info();
		load_test_standard_info();
	}
	else
	{
		tempStr[0] = CMD_SET_STANDARD;
		tempStr[1] = 0;
		rs232_write_bytes(&tempStr[0], 2);
	}
}

//----------------------------------------------------------------------------------
void set_units(st_units new_units)
//----------------------------------------------------------------------------------
{
	if (new_units < st_units_max)
	{
		tempStr[0] = CMD_SET_UNITS;
		tempStr[1] = 1;
		rs232_write_bytes(&tempStr[0], 2);

		units = new_units;
		save_calibration_info();
	}
	else
	{
		tempStr[0] = CMD_SET_UNITS;
		tempStr[1] = 0;
		rs232_write_bytes(&tempStr[0], 2);
	}
}

//----------------------------------------------------------------------------------
void get_decimal_places(void)
//----------------------------------------------------------------------------------
{
	tempStr[0] = CMD_DECIMAL_PLACES;
	tempStr[1] = gUnitsDecimals[units];
	rs232_write_bytes(&tempStr[0], 2);
}

//----------------------------------------------------------------------------------
void do_start_test(unsigned int8 channel)
//----------------------------------------------------------------------------------
{
	tempStr[0] = CMD_START_TEST;
	tempStr[1] = 1;
	rs232_write_bytes(&tempStr[0], 2);

	initialise_test(channel);
	testing = true;
	reset_solenoids(channel);
	update_test_status();
}

//----------------------------------------------------------------------------------
void get_max_samples(void)
//----------------------------------------------------------------------------------
{
	unsigned int16 ver;

	ver = test_standard_info.total_samples;
	tempStr[0] = CMD_MAX_SAMPLES;
	tempStr[1] = (unsigned char) MAKE8(ver, 1);
	tempStr[2] = (unsigned char) MAKE8(ver, 0);
	rs232_write_bytes(&tempStr[0], 3);
}

//----------------------------------------------------------------------------------
void get_stabilisation_seconds(int8 channel)
//----------------------------------------------------------------------------------
{
	unsigned int16 ver;

	ver = stabilise[channel];
	tempStr[0] = CMD_GET_STAB_SECS;
	tempStr[1] = (unsigned char) MAKE8(ver, 1);
	tempStr[2] = (unsigned char) MAKE8(ver, 0);
	rs232_write_bytes(&tempStr[0], 3);
}

//----------------------------------------------------------------------------------
void get_max_stabilisation_seconds(void)
//----------------------------------------------------------------------------------
{
	unsigned int16 ver;

	ver = test_standard_info.stabilise_time_seconds;
	tempStr[0] = CMD_GET_MAX_STAB_SECS;
	tempStr[1] = (unsigned char) MAKE8(ver, 1);
	tempStr[2] = (unsigned char) MAKE8(ver, 0);
	rs232_write_bytes(&tempStr[0], 3);
}

// =============================================================
#endif